package com.phm.netty.service.impl;

import com.alibaba.fastjson2.JSON;
import com.phm.common.utils.bean.BeanUtils;
import com.phm.manage.domain.OrderInfo;
import com.phm.manage.domain.common.OrderXmlVO;
import com.phm.manage.enums.OrderStatus;
import com.phm.manage.service.IOrderInfoService;
import com.phm.manage.service.ISortieService;
import com.phm.manage.util.XMLParserWithJAXB;
import com.phm.netty.enums.MessageEnum;
import com.phm.netty.service.IProcessService;
import com.phm.netty.domain.Message;
import com.phm.system.service.ISysDictDataService;
import com.phm.system.service.ISysDictTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.util.Objects;

import static com.phm.netty.enums.MessageEnum.GET_DATA_BY_ONBOARD;

/**
 * @Description ProcessService
 * @Author WGK
 * @Date 2023/9/22 9:54
 */

@Service
public class ProcessService implements IProcessService {

    @Autowired
    private IOrderInfoService orderInfoService;

    @Autowired
    private ISysDictTypeService dictTypeService;

    @Autowired
    private ISortieService sortieService;

    @Override
    public Message orderHandle(Message message) {
        MessageEnum type = MessageEnum.getStructureEnum(message);
        OrderXmlVO orderXmlVO = XMLParserWithJAXB.xmlToObject(message.getData());
        OrderInfo orderInfo = new OrderInfo();
        BeanUtils.copyBeanProp(orderInfo, orderXmlVO);
        orderInfo.setContent(JSON.toJSONString(message.getData()));
        // set待执行状态
        orderInfo.setStatus(OrderStatus.S_0.getCode());
        orderInfoService.insertOrderInfo(orderInfo);
        Message msg = new Message();
        msg.setType(MessageEnum.SUCCESS.getType()).setData(MessageEnum.SUCCESS.getContent());
        switch (Objects.requireNonNull(type)) {
            case GET_DATA_BY_ONBOARD:
                // 机载数据获取指令
            case GET_DATA_BY_CHAIN:
                // 数据链与数据仿真数据获取指令
            case FALSE_ALARM:
                // 机载故障虚警抑制指令
            case ISOLATE:
                // 机载故障深度隔离指令
            case GROUND_FAULT:
                // 地面故障诊断指令
            case FAILURE_PREDICTION:
                // 故障预测指令
            case GET_SORTIE:
                // 获取架次信息
            case NTP:
                // 校时
            default:
                // 默认
                return msg;
        }
    }
}
